from error import RNA2dError
from ss_writer import SSWriter


class DBNWriter(SSWriter):
    def __init__(self):
        super(DBNWriter, self).__init__()
        self.symbols = {1: ('(', ')'),
                        2: ('[', ']'),
                        3: ('{', '}'),
                        4: ('<', '>'),
                        5: ('A', 'a')}

    def _get_level(self, ss, pair):
        if len(pair) == 2:
            resi, resj = pair
        else:
            resi, resj, _ = pair
        for level in range(1, len(self.symbols) + 1):
            score = 0
            overlap = False
            for i in range(resi, resj - 1):
                if ss[i] == self.symbols[level][0]:
                    score += 1
                elif ss[i] == self.symbols[level][1]:
                    score -= 1
                if score < 0:
                    overlap = True
                    break
            if not overlap:
                return level
        return -1

    def pairs_to_dbn(self, pairs, nres):
        ss = ['.'] * nres
        for pair in pairs:
            if len(pair) == 2:
                resi, resj = pair
            else:
                resi, resj, _ = pair
            resi = resi - 1
            resj = resj - 1
            if resi > resj:
                resi, resj = resj, resi
            if ss[resi] == '.' and ss[resj] == '.':
                level = self._get_level(ss, pair)
                if level == -1:
                    raise RNA2dError('Secondary structure exceeds maximum levels of DBN')
                ss[resi] = self.symbols[level][0]
                ss[resj] = self.symbols[level][1]
        return ''.join(ss)

    def write_str(self, data):
        result = []
        name = self.get_name(data)
        seq, pairs, _ = self.split_data(data)
        nres = len(seq)
        result.append('> {:s} Generated by 2dRNA\n'.format(name))
        result.append('{:s}\n'.format(seq))
        dbn = self.pairs_to_dbn(pairs, nres)
        result.append('{:s}\n'.format(dbn))
        return "".join(result)

    def write(self, data, output):
        fh = self.get_output_handler(output)
        fh.write(self.write_str(data))


if __name__ == '__main__':
    from di_parser import DIParser
    import sys

    assert len(sys.argv) > 2
    seq_file, di_file = sys.argv[1], sys.argv[2]
    parser = DIParser()
    with open(seq_file) as f:
        seq = f.readline().strip()
    di_values = parser.parse(di_file, value=False)
    data = {'seq': seq,
            'pairs': di_values,
            'name': 'test'}
    writer = DBNWriter()
    writer.write(data, 'test.dbn')
