from __future__ import print_function

import sys

from error import RNA2dError


class DIParser(object):
    def __init__(self, min_loop=2):
        self.values = []
        self.min_loop = min_loop

    def _parse_line(self, line):
        fields = line.strip().split()
        n_fields = len(fields)
        if n_fields == 2:
            resi, resj = int(fields[0]), int(fields[1])
            value = 0.0
        elif n_fields == 3:
            resi, resj = int(fields[0]), int(fields[1])
            value = float(fields[2])
        elif n_fields == 4:
            resi, resj = int(fields[0]), int(fields[2])
            value = 0.0
        elif n_fields == 5:
            resi, resj = int(fields[0]), int(fields[2])
            value = float(fields[4])
        else:
            raise RNA2dError("Invalid DI line: '{:s}'".format(line))
        if resi > resj:
            resi, resj = resj, resi
        if resj - resi <= self.min_loop:
            print('WARNING: ignore the di record line with abs(i-j) <= 4', file=sys.stderr)
            return None
        return resi, resj, value

    def parse_str(self, input_str, value=True):
        lines = input_str.strip().split('\n')
        for line in lines:
            di_record = self._parse_line(line)
            if not di_record:
                continue
            resi, resj, di = di_record
            if value:
                self.values.append((resi, resj, di))
            else:
                self.values.append((resi, resj))
        if value:
            self.values.sort(key=lambda x: x[2], reverse=True)
        return None,self.values

    def parse(self, input_file, value=True):
        with open(input_file, 'r') as f:
            lines = f.readlines()
        return self.parse_str("".join(lines), value)
