from ss_writer import SSWriter


class DIWriter(SSWriter):
    def __init__(self):
        super(DIWriter, self).__init__()
        self.fmt = '{:5d}  {:s}{:5d}  {:s}{:8.3f}\n'

    def split_data(self, data):
        seq, pairs, _ = super(DIWriter, self).split_data(data)
        if 'di' not in data:
            self.fmt = '{:5d}  {:s}{:5d}  {:s}\n'
            return seq, pairs, None
        else:
            return seq, pairs, data['di']

    def _remove_duplicated_pairs(self, pairs):
        pairs_set = set()
        for pair in pairs:
            i, j = pair
            if i > j:
                i, j = j, i
            pairs_set.add((i, j))
        return sorted(pairs_set, key=lambda x: x[0])

    def write_str(self, data):
        result = []
        seq, pairs, di = self.split_data(data)
        pairs = self._remove_duplicated_pairs(pairs)
        if di:
            for (i, j), v in zip(pairs, di):
                resi, resj = seq[i - 1], seq[j - 1]
                result.append(self.fmt.format(i, resi, j, resj, v))
        else:
            for i, j in pairs:
                resi, resj = seq[i - 1], seq[j - 1]
                result.append(self.fmt.format(i, resi, j, resj))
        return "".join(result)

    def write(self, data, output):
        fh = self.get_output_handler(output)
        fh.write(self.write_str(data))
