from bpseq_parser import BPSeqParser
from bpseq_writer import BPSeqWriter
from ct_parser import CTParser
from ct_writer import CTWriter
from dbn_parser import DBNParser
from dbn_writer import DBNWriter
from di_parser import DIParser
from di_writer import DIWriter
from error import RNA2dError
from mfold import Mfold
from rna_fold import RNAfold
from rna_structure import RNAStructure
from seq_writer import SeqWriter


class Factory(object):
    @staticmethod
    def make_ss_writer(fmt):
        if fmt == 'dbn':
            return DBNWriter()
        elif fmt == 'ct':
            return CTWriter()
        elif fmt == 'bpseq':
            return BPSeqWriter()
        elif fmt == 'di':
            return DIWriter()
        else:
            raise RNA2dError('invalid output format, only dbn, ct, bpseq and di is valid!')

    @staticmethod
    def make_ss_parser(fmt):
        if fmt == 'dbn':
            return DBNParser()
        elif fmt == 'ct':
            return CTParser()
        elif fmt == 'bpseq':
            return BPSeqParser()
        elif fmt == 'di':
            return DIParser()
        else:
            raise RNA2dError('invalid input format, only dbn, ct bpseq and di is valid!')

    @staticmethod
    def make_folder(args):
        if args.method == 'mfold':
            return Mfold(args)
        elif args.method == 'RNAfold':
            return RNAfold(args)
        elif args.method == 'RNAstructure':
            return RNAStructure(args)
        else:
            raise RNA2dError('Invalid RNA folding method')

    @staticmethod
    def make_seq_writer():
        return SeqWriter()
