import os


def get_file_ext(file_name):
    return os.path.splitext(file_name)[1][1:]


def get_prefix(seq_file):
    if seq_file.find('.') < 0:
        return seq_file
    else:
        return ".".join(seq_file.split(".")[:-1])


def pairs_to_aux(pairs):
    sorted_pairs = sorted(pairs, key=lambda x: x[0])
    if len(sorted_pairs[0]) == 3:
        sorted_pairs = [(p[0], p[1]) for p in sorted_pairs]
    last_pair = (-1, -1)
    aux_records = []
    pair_count = 0
    starti = startj = 0
    for pair in sorted_pairs:
        resi, resj = pair
        last_resi, last_resj = last_pair
        if resi == last_resi + 1 and resj == last_resj - 1:
            pair_count += 1
        else:
            if pair_count > 0:
                aux_records.append((starti, startj, pair_count))
            starti, startj = resi, resj
            pair_count = 1
        last_pair = pair
    aux_records.append((starti, startj, pair_count))
    return aux_records


def find_min_dg_start(ct_file):
    dg_lines = []
    with open(ct_file, 'r') as fh:
        for idx, line in enumerate(fh):
            if line.find("dG =") > 0:
                dg_lines.append((line, idx))
    dg_lines.sort(key=lambda x: float(x[0].strip().split()[3]))
    return dg_lines[0][1]


def ct_to_pairs(ct_file):
    start = find_min_dg_start(ct_file)
    pairs = []
    with open(ct_file, 'r') as fh:
        lines = fh.readlines()
        for line in lines[start + 1:]:
            if line.find("dG =") > 0:
                break
            fields = line.strip().split()
            res_i, res_j = int(fields[0]), int(fields[4])
            if res_i < res_j:
                pairs.append((res_i, res_j))
    return pairs


if __name__ == '__main__':
    pairs = [(1, 92),
             (2, 91),
             (3, 89),
             (4, 88),
             (11, 36),
             (12, 35),
             (16, 31),
             (17, 30),
             (18, 29),
             (19, 28),
             (20, 27),
             (21, 26),
             (40, 84),
             (41, 83),
             (42, 82),
             (44, 81),
             (45, 80),
             (46, 79),
             (47, 76),
             (48, 75),
             (49, 74),
             (50, 73),
             (51, 72),
             (54, 71),
             (55, 70),
             (56, 69),
             (57, 68)]
    print(pairs_to_aux(pairs))
