import glob
import subprocess

from folder import Folder
from helper import *
from ss_transformer import SSTransformer


class Mfold(Folder):
    def __init__(self, args):
        super(Mfold, self).__init__(args)
        self.set_command('mfold')
        self.prefix = self.get_prefix()
        self.seq_basename = os.path.basename(self.temp_seq)

    def get_prefix(self):
        if self.args.seq_file:
            return get_prefix(self.args.seq_file)
        else:
            return get_prefix(self.args.ss_file)

    def write_cstr(self, aux):
        fh = open(self.temp_cstr, 'w')
        for stem in aux:
            fh.write("F {0[0]:d} {0[1]:d} {0[2]:d}\n".format(stem))
        fh.close()

    def __del__(self):
        files = glob.glob(self.seq_basename + "*")
        for f in files:
            os.remove(f)

    def run(self):
        from factory import Factory
        pairs, pk_pairs = SSTransformer.remove_pk(self.pairs)
        self.write_cstr(pairs_to_aux(pairs))
        self.write_seq()
        cmd = [self.command, "SEQ=" + self.temp_seq, "AUX=" + self.temp_cstr]
        with open(os.devnull, 'w') as devnull:
            subprocess.check_call(cmd, stderr=devnull, stdout=devnull)
        mfold_out = self.seq_basename + ".ct"
        parser = Factory.make_ss_parser('ct')
        _, pairs = parser.parse(mfold_out)
        pairs = Folder.add_pk_to_pairs(pairs, pk_pairs)
        self.write_out(pairs)
