import os

from error import RNA2dError
from rna import RNA
from ss_writer import SSWriter
from seq_parser import SeqParser


class SSTransformer(SSWriter):
    def __init__(self, **args):
        super(SSTransformer, self).__init__()
        self.args = args
        self._set_no_pk()
        self.formats = RNA.ss_formats
        self.args = args
        self._set_input()
        self.seq, self.pairs = self._read_files()
        self.data = self._get_data()

    def _set_no_pk(self):
        if "no_pk" in self.args:
            self.no_pk = self.args["no_pk"]
        else:
            self.no_pk = False

    def _get_data(self):
        if self.no_pk:
            self.pairs, _ = SSTransformer.remove_pk(self.pairs)
        return {'seq': self.seq, 'pairs': self.pairs}

    def _read_files(self):
        from factory import Factory
        parser = Factory.make_ss_parser(self.input_fmt)
        if 'input_file' in self.args:
            seq, pairs = parser.parse(self.input_file)
        else:
            seq, pairs = parser.parse_str(self.input)
        if not seq:
            seq_file = self.args["seq_file"]
            seq_parser = SeqParser()
            seq = seq_parser.parse(seq_file)
        return seq, pairs

    def _set_input(self):
        if 'input_file' in self.args:
            self.input_file = self.args['input_file']
        elif 'input' in self.args:
            self.input = self.args['input']
        else:
            raise RNA2dError('Missing input or input_file parameter')
        if 'input_fmt' in self.args:
            self.input_fmt = self.args['input_fmt']
        else:
            self.input_fmt = self.get_fmt()

    @staticmethod
    def bp_interact(pair_i, pair_j):
        if len(pair_i) == 3:
            pair_i = pair_i[:2]
            pair_j = pair_j[:2]
        if pair_i[0] > pair_j[0]:
            pair_i, pair_j = pair_j, pair_i
        i_left, i_right = pair_i
        if i_left > i_right:
            i_left, i_right = i_right, i_left
        j_left, j_right = pair_j
        if j_left > j_right:
            j_left, j_right = j_right, j_left
        if i_left < j_left < i_right < j_right:
            return True
        else:
            return False

    @staticmethod
    def remove_pk(pairs):
        result = []
        result_pk = []
        pair_indices = []
        pk_indices = []
        pairs.sort(key=lambda x: x[0])
        n_pairs = len(pairs)
        for i in range(n_pairs):
            pair_i = pairs[i]
            should_preserve = True
            for j in pair_indices:
                pair_j = pairs[j]
                if SSTransformer.bp_interact(pair_i, pair_j):
                    should_preserve = False
                    break
            if should_preserve:
                pair_indices.append(i)
            else:
                pk_indices.append(i)
        for i in pair_indices:
            result.append(pairs[i])
        for i in pk_indices:
            result_pk.append(pairs[i])
        return result, result_pk

    def get_fmt(self):
        if 'input_file' not in self.args:
            raise RNA2dError('Cannot determine format of input')
        _, ext = os.path.splitext(self.input_file)
        ext = ext[1:]
        if ext not in self.formats:
            raise RNA2dError('Cannot determine format of input')
        else:
            return ext

    def write_str(self, fmt):
        from factory import Factory
        writer = Factory.make_ss_writer(fmt)
        return writer.write_str(self.data)

    def write(self, output, fmt):
        fh = self.get_output_handler(output)
        fh.write(self.write_str(fmt))
