import subprocess

from dbn_writer import DBNWriter
from folder import Folder
from ss_transformer import SSTransformer


class RNAfold(Folder):
    def __init__(self, args):
        super(RNAfold, self).__init__(args)
        self.set_command('RNAfold')

    def write_cstr(self, pairs):
        writer = DBNWriter()
        data = {'seq': self.seq,
                'pairs': pairs}
        writer.write(data, self.temp_cstr)

    def write_out(self, pairs):
        from factory import Factory
        writer = Factory.make_ss_writer(self.out_fmt)
        data = {'seq': self.seq,
                'pairs': pairs}
        writer.write(data, self.output)

    def run(self):
        from factory import Factory
        pairs, pk_pairs = SSTransformer.remove_pk(self.pairs)
        self.write_cstr(pairs)
        cmd = [self.command, "-C", self.temp_cstr, "--enforceConstraint"]
        output = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
        output = output.decode('utf-8')
        parser = Factory.make_ss_parser('dbn')
        _, pairs = parser.parse_str(output)
        pairs = RNAfold.add_pk_to_pairs(pairs, pk_pairs)
        self.write_out(pairs)
