import os
import subprocess
import sys
import tempfile

from error import RNA2dError
from folder import Folder
from ss_transformer import SSTransformer


class RNAStructure(Folder):
    def __init__(self, args):
        super(RNAStructure, self).__init__(args)
        self.set_command('Fold')
        if 'DATAPATH' not in os.environ:
            raise RNA2dError('Please set DATAPATH for RNAstructure')

    def write_cstr(self, pairs):
        self.temp_cstr = tempfile.mktemp()
        with open(self.temp_cstr, 'w') as f:
            f.write("DS:\n-1\nSS:\n-1\nMod:\n-1\nPairs:\n")
            for pair in pairs:
                f.write('{:d} {:d}\n'.format(pair[0], pair[1]))
            f.write("-1 -1\nFMN:\n-1\nForbids:\n-1 -1")

    def run(self):
        pairs, pk_pairs = SSTransformer.remove_pk(self.pairs)
        self.write_cstr(pairs)
        self.write_seq()
        if self.output == sys.stdout:
            cmd = [self.command, self.temp_seq, "-", "-mfe", "-C", self.temp_cstr]
            output = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
            output = output.decode('utf-8')
            transformer = SSTransformer(input=output, input_fmt='ct')
        else:
            temp_output = tempfile.mktemp()
            cmd = [self.command, self.temp_seq, temp_output, "-c", self.temp_cstr]
            _ = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
            transformer = SSTransformer(input_file=temp_output, input_fmt='ct')
        transformer.write(self.output, self.out_fmt)
